/* Copyright 2008 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: MainState.h,v 1.7 2008-04-20 23:44:23 tonic Exp $
 * $Revision: 1.7 $
 */

#ifndef MAINSTATE_H_INCL
#define MAINSTATE_H_INCL


#include <vector>

//using namespace std;
using namespace irr;
using namespace scene;
using namespace core;
using namespace video;


enum ENEMY_TYPE
{
    ET_TETRAHEDRON = 0,
    ET_CUBE,
    ET_DODECAHEDRON,
    ET_OCTAHEDRON,
    ET_ICOSAHEDRON,
    ET_OCTOTOAD,
    ET_COUNT //
};


class MainState : public AppState
{
public:

    MainState();
    virtual ~MainState();

    // From AppState:

    virtual bool init();
    virtual void deinit();

    virtual bool update(UI32 time);
    virtual void render(irr::IrrlichtDevice *device);

    virtual void onActivate(UI32 time);
    virtual void onDeactivate() {}

    virtual bool onKeyDown(const irr::SEvent::SKeyInput &event);
    virtual bool onKeyUp(const irr::SEvent::SKeyInput &event);
    virtual bool onMouseDown(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseUp(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseMotion(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseWheel(const irr::SEvent::SMouseInput &event);
    // Irr is lacking focus lost/gain events..
    //virtual void onFocusLost(const SDL_ActiveEvent &event) {}
    //virtual void onFocusGain(const SDL_ActiveEvent &event) {}

protected:

    enum IMAGE_ID
    {
        IMG_BUTTON = 0,
        IMG_BUTTON_HOT,
        IMG_BUTTON_ACTIVE,
        IMG_BUTTON_HOT_ACTIVE
    };

    struct RoutePoint
    {
        vector3df cameraPos;
        vector3df objPos;
    };

    struct EnemyScriptUnit
    {
        EnemyScriptUnit(u32 spawnTime, vector3df spawnPos, ENEMY_TYPE enemyType, f32 ang) :
            time(spawnTime), pos(spawnPos), type(enemyType), angle(ang) {}
        u32 time;
        vector3df pos;
        ENEMY_TYPE type;
        f32 angle;

        bool operator<(const EnemyScriptUnit &rhs) { return time < rhs.time; }
    };

    struct Enemy
    {
        Enemy(ENEMY_TYPE t, vector3df position, f32 ang, IAnimatedMeshSceneNode *node, f32 radius) :
            type(t), pos(position), angle(ang), sceneNode(node), r(radius), deathCount(-1), age(0) {}
        ENEMY_TYPE type;
        vector3df pos;
        f32 angle;
        IAnimatedMeshSceneNode *sceneNode;
        f32 r;
        s32 deathCount; // enemy is dead when this is 0
        s32 age;
    };

    struct Bullet
    {
        vector3df pos;
        vector3df speed;
        u32 framesLeft;
        ISceneNode *sceneNode;
    };

    vector3df getSplinePosition(u32 time, f32 speed, f32 tightness, bool target = false);

    void runEnemyScriptSpawns();
    void updateEnemies();
    void updateBullets();
    void killEnemy(s32 index, bool getScore, bool oldAge);
    void moveTowardsPlayer(vector3df &ioPos, f32 speed);

    App *mApp;

    IMGUI mIMGUI;
    IMGUIImageButtonStyle *mButtonStyle;

    Font *mFont;
    std::vector<Image *> mImages;
    C8 *mLabelText;

    IrrlichtDevice *mDevice;
    IVideoDriver *mVideo;
    ISceneManager *mSmgr;

    ICameraSceneNode *mCamera;

    vector2di mLastMousePos;

    bool mTitleUIActive;
    bool mInGame;
    bool mEndingUIActive;

    ISceneNode *mPlayerParentSceneNode;
    IAnimatedMeshSceneNode *mPlayerSceneNode;
    ISceneNode *mArrowSceneNode;

    f32 mAimAngle;

    s32 mScore;

    u32 mCurrentEnemyScriptIndex;
    std::vector<RoutePoint> mRoutePoints;
    std::vector<EnemyScriptUnit> mEnemyScript;  // must be sorted by time
    std::vector<Enemy> mEnemies;
    std::vector<Bullet> mBullets;

    IAnimatedMesh *mEnemyMeshes[ET_COUNT];
    IAnimatedMesh *mEnemyShadowMeshes[ET_COUNT];
    IMesh *mBulletMesh;
    SMaterial mBulletMaterial;
};


#endif // !MAINSTATE_H_INCL
