/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUI.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2IMGUI_H_INCL
#define T2IMGUI_H_INCL


namespace turska
{
    enum IMGUI_AUTO_ID
    {
        IMGUI_AI_COUNTER      = -1
    };
    
    enum IMGUI_ACTIVE_WIDGET_STATUS
    {
        IMGUI_AWS_NONE        = -1,
        IMGUI_AWS_UNAVAILABLE = -2
    };

    //! Default widget states.
    enum IMGUI_WIDGET_STATE
    {
        IMGUI_WS_DEFAULT     = 0x0,
        IMGUI_WS_HOT         = 0x1,
        IMGUI_WS_ACTIVE      = 0x2,
        IMGUI_WS_HOT_ACTIVE  = 0x3,
        IMGUI_WS_COUNT, // count of states

        //! Bitmask for checking if state is one of "hot" states.
        IMGUI_WS_MASK_HOT    = 0x1,
        //! Bitmask for checking if state is one of "active" states.
        IMGUI_WS_MASK_ACTIVE = 0x2
    };

    //! Widget orientation (used e.g. by slider).
    enum IMGUI_WIDGET_ORIENTATION
    {
        IMGUI_WO_HORIZONTAL = 0,
        IMGUI_WO_VERTICAL
    };

    class IMGUIButtonStyle;
    class IMGUISliderStyle;

    //! Immediate-mode graphical user interface (IMGUI).
    class IMGUI
    {
    public:

        IMGUI();
        virtual ~IMGUI();

        virtual void begin(I16 groupId = 0);
        virtual void end();

        // If width or height is 0, it's assumed that the style object knows better.
        // x,y refers to top-left corner

        virtual bool button(REAL x, REAL y, I32 id = IMGUI_AI_COUNTER,
                            IMGUIButtonStyle *buttonStyle = 0,
                            REAL width = 0, REAL height = 0,
                            const C8 *text = 0, Image *image = 0);

        virtual void slider(REAL x, REAL y, IMGUI_WIDGET_ORIENTATION axis,
                            I32 &value, I32 minValue = 0, I32 maxValue = 100,
                            I32 id = IMGUI_AI_COUNTER,
                            IMGUISliderStyle *sliderStyle = 0,
                            REAL width = 0, REAL height = 0);
        virtual void slider(REAL x, REAL y, IMGUI_WIDGET_ORIENTATION axis,
                            REAL &value, REAL minValue = 0, REAL maxValue = 1,
                            I32 id = IMGUI_AI_COUNTER,
                            IMGUISliderStyle *sliderStyle = 0,
                            REAL width = 0, REAL height = 0);

        virtual void onKeyDown(const KeyEvent &key);
        virtual void onMouseDown();
        virtual void onMouseUp();
        virtual void onMouseMotion(I32 x, I32 y);

    protected:

        I32 defaultWidgetStateLogic(I32 id, bool hotNow);
        I32 buttonLogic(I32 id, IMGUIButtonStyle *buttonStyle,
                        REAL x, REAL y, REAL width, REAL height,
                        const C8 *text, Image *image);
        bool isButtonClicked(I32 id);

        I32 mMouseX;
        I32 mMouseY;
        bool mMouseDown;
        I32 mHotItemId;
        I32 mActiveItemId;
        I32 mAutoIdCounter;
    };

} // namespace turska


#endif // !T2IMGUI_H_INCL
