/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUIButtonStyle.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2IMGUIBUTTONSTYLE_H_INCL
#define T2IMGUIBUTTONSTYLE_H_INCL


namespace turska
{
    class IMGUIButtonStyle
    {
    public:
        virtual ~IMGUIButtonStyle() {}

        //! Returns true if x,y is inside given rectangle.
        /*! The rectangle is defined by top-left point 0,0 (inclusive) and
         *  bottom-right point width,height (exclusive). If width or height
         *  is 0 then the style object should either know the proper size
         *  itself, or it should find out the width and height using either
         *  given text or given image.
         * \param x X-coordinate to check for.
         * \param y Y-coordinate to check for.
         * \param width width of the button area, or 0 if style should know it.
         * \param height height of the button area, or 0 if style should know it.
         */
        virtual bool isInside(REAL x, REAL y, REAL width, REAL height,
                              const C8 *text, Image *image) = 0;

        
        //! Renders a button.
        /*!
         * \param widgetState one of the sates from IMGUI_WIDGET_STATE.
         * \param x X-coordinate of top-left corner.
         * \param y Y-coordinate of top-left corner.
         * \param width width of the button, or 0 if style should know it.
         * \param height height of the button, or 0 if style should know it.
         * \param text text to print, or 0 if none given.
         * \param image item to draw, or 0 if none given.
         */
        virtual void render(I32 widgetState, REAL x, REAL y,
                            REAL width, REAL height,
                            const C8 *text, Image *image) = 0;
    };

} // namespace turska


#endif // !T2IMGUIBUTTONSTYLE_H_INCL
