/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUIImageButtonStyle.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2IMGUIIMAGEBUTTONSTYLE_H_INCL
#define T2IMGUIIMAGEBUTTONSTYLE_H_INCL


namespace turska
{
    //! Image button with fixed extents defined by the image size.
    /*! The width and height are ignored when there's an image to use.
     */
    class IMGUIImageButtonStyle : public IMGUIButtonStyle
    {
    public:
        IMGUIImageButtonStyle(Font *font, Image *imageUp, Image *imageHot,
                              Image *imageActive, Image *imageHotDown);

        // From IMGUIButtonStyle:

        virtual bool isInside(REAL x, REAL y, REAL width, REAL height,
                              const C8 *text, Image *image);

        virtual void render(I32 widgetState, REAL x, REAL y,
                            REAL width, REAL height,
                            const C8 *text, Image *image);

        // Local

        void setScale(REAL scale) { mScale = scale; }
        void setButtonColor(const Color &color) { mButtonColor = color; }
        void setItemColor(const Color &color) { mItemColor = color; }
        void setFont(Font *font) { mFont = font; }

        void setShadowMultiplierColor(const Color &color) { mShadowMultiplierColor = color; }
        void setShadowEnabled(bool enabled) { mShadowEnabled = enabled; }
        void setShadowOffset(REAL xOffset, REAL yOffset) { mShadowXOffset = xOffset; mShadowYOffset = yOffset; }
        void setButtonDownItemOffset(REAL xOffset, REAL yOffset) { mButtonDownItemXOffset = xOffset; mButtonDownItemYOffset = yOffset; }

        REAL getWidth() { return mScale * mImageWidth; }
        REAL getHeight() { return mScale * mImageHeight; }

    protected:
        REAL mScale;
        Color mButtonColor;
        Color mItemColor;
        Color mShadowMultiplierColor;
        Image *mImages[IMGUI_WS_COUNT];
        UI32 mImageWidth, mImageHeight;
        Font *mFont;
        bool mShadowEnabled;
        REAL mShadowXOffset, mShadowYOffset;
        REAL mButtonDownItemXOffset, mButtonDownItemYOffset;
    };

} // namespace turska


#endif // !T2IMGUIIMAGEBUTTONSTYLE_H_INCL
