/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUISliderStyle.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2IMGUISLIDERSTYLE_H_INCL
#define T2IMGUISLIDERSTYLE_H_INCL


namespace turska
{
    class IMGUISliderStyle
    {
    public:
        virtual ~IMGUISliderStyle() {}

        //! Returns true if x,y is inside given rectangle.
        /*! The rectangle is defined by top-left point 0,0 (inclusive) and
         *  bottom-right point width,height (exclusive). If width or height
         *  is 0 then the style object should know the proper size by itself.
         */
        virtual bool isInside(REAL x, REAL y, REAL width, REAL height) = 0;

        
        //! Returns integer value of point in slider, see definition of isInside.
        /*! \todo more detailed documentation
         */
        virtual I32 getIntValue(IMGUI_WIDGET_ORIENTATION axis, I32 minValue, I32 maxValue,
                                REAL x, REAL y, REAL width, REAL height) = 0;
        
        //! Returns integer value of point in slider, see definition of isInside.
        /*! \todo more detailed documentation
         */
        virtual REAL getRealValue(IMGUI_WIDGET_ORIENTATION axis, REAL minValue, REAL maxValue,
                                  REAL x, REAL y, REAL width, REAL height) = 0;


        //! Renders a slider.
        /*! x,y refers to top-left corner
         * \todo more detailed documentation
         */
        virtual void render(I32 widgetState, IMGUI_WIDGET_ORIENTATION axis,
                            I32 value, I32 minValue, I32 maxValue,
                            REAL x, REAL y, REAL width, REAL height) = 0;
        
        //! Renders a slider.
        /*! x,y refers to top-left corner
         * \todo more detailed documentation
         */
        virtual void render(I32 widgetState, IMGUI_WIDGET_ORIENTATION axis,
                            REAL value, REAL minValue, REAL maxValue,
                            REAL x, REAL y, REAL width, REAL height) = 0;
    };

} // namespace turska


#endif // !T2IMGUISLIDERSTYLE_H_INCL
