/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2ImageIrr.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2IMAGEIRR_H_INCLUDED
#define T2IMAGEIRR_H_INCLUDED


namespace irr
{
    namespace video
    {
        class ITexture;
    }
}

namespace turska
{
    class GraphicsIrr;

    //! Implementation of Turska Image which uses Irrlicht.
    /*! \note In this implementation the IF_COLOR and IF_ALPHA flags
     *        are always assumed to be enabled.
     */
    class ImageIrr : public turska::Image
    {
    public:

        //! Constructs a new ImageIrr from given texture and U,V coordinates.
        /*!
         * \param width original width of the image in pixels.
         * \param height original height of the image in pixels.
         * \param flags initial flags to use for the image.
         * \param texture irr::video::ITexture to be used as image. Texture is
         *                grabbed and dropped when this object is deleted.
         * \param uv array of 2 vectors describing the top-left and
         *           bottom-right U,V coordinates.
         */
        ImageIrr(GraphicsIrr *graphics, UI32 width, UI32 height,
                 UI32 flags, irr::video::ITexture *texture,
                 const irr::core::vector2df *uv);
        virtual ~ImageIrr();

        // From Image:

        virtual void setColor(const Color &color);

        virtual void setTargetSize(UI32 width, UI32 height) { setTargetSize((REAL)width, (REAL)height); }
        virtual void setTargetSize(REAL width, REAL height) { mTargetWidth = width; mTargetHeight = height; }
        
        virtual void setAngle(REAL angle) { mAngle = angle; }

        virtual void setScale(REAL scale) { mScaleX = mScaleY = scale; }
        virtual void setScale(REAL scaleX, REAL scaleY) { mScaleX = scaleX; mScaleY = scaleY; }
        
        virtual void setPivot(REAL centerX, REAL centerY) { mPivotX = centerX + mNegHalfWidth; mPivotY = centerY + mNegHalfHeight; }

        virtual void draw(I32 x, I32 y) { draw((REAL)x, (REAL)y); }
        virtual void draw(REAL x, REAL y) { draw(mDrawMode, mFlags, x, y); }
        virtual void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, I32 x, I32 y) { draw(drawMode, flags, (REAL)x, (REAL)y); }
        virtual void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, REAL x, REAL y) { draw(drawMode, flags, x, y, mColor); }
        virtual void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, REAL x, REAL y, const Color &color);

        // ImageIrr additions:

        inline irr::video::ITexture * getIrrTexture() const { return mTexture; }
        inline irr::core::vector2df * getUV() { return mUV; }

    protected:

        void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, F32 x, F32 y,
                  const irr::video::SColor &color);

        GraphicsIrr *mGraphics;
        irr::video::IVideoDriver *mDriver;
        irr::video::ITexture *mTexture;
        irr::core::vector2df mUV[2];
        const F32 mNegHalfWidth;
        const F32 mNegHalfHeight;
        const F32 mPosHalfWidth;
        const F32 mPosHalfHeight;
        irr::video::SColor mColor;
        F32 mTargetWidth;
        F32 mTargetHeight;
        F32 mAngle;
        F32 mScaleX, mScaleY;
        F32 mPivotX, mPivotY;
    }; // ImageIrr
} // namespace turska


#endif // !T2IMAGEIRR_H_INCLUDED
