/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2ImageSDL.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2IMAGESDL_H_INCLUDED
#define T2IMAGESDL_H_INCLUDED


struct SDL_Surface;

namespace turska
{
    //! Implementation of Turska Image which uses SDL surfaces.
    /*! \note This is unfinished and partial implementation.
     * \todo support for color, rotation, scale, etc...
     */
    class ImageSDL : public turska::Image
    {
    public:

        //! Constructs a new ImageSDL from given SDL_Surface.
        /*!
         * \param surface image to use, ownership is transferred.
         * \param flags initial flags to use for the image.
         */
        ImageSDL(SDL_Surface *surface, UI32 flags);
        //! Destructor deletes the surface.
        virtual ~ImageSDL();

        // From turska::Image:

        virtual void setColor(const Color &color);

        virtual void setTargetSize(UI32 width, UI32 height)
        {
            mTargetWidth = width; mTargetHeight = height;
        }

        virtual void setTargetSize(REAL width, REAL height)
        {
            if (width < 0) width = 0;
            if (height < 0) height = 0;
            setTargetSize((UI32)(I32)width, (UI32)(I32)height);
        }


        virtual void setAngle(REAL angle) { /* TODO mAngle = angle;*/ }
        virtual void setScale(REAL scale) { /* TODO mScaleX = mScaleY = scale; */ }
        virtual void setScale(REAL scaleX, REAL scaleY) { /* TODO mScaleX = scaleX; mScaleY = scaleY; */ }
        virtual void setPivot(REAL centerX, REAL centerY) { /* TODO - set pivot */ }


        virtual void draw(I32 x, I32 y) { draw(mDrawMode, mFlags, x, y); }
        virtual void draw(REAL x, REAL y) { draw((I32)x, (I32)y); }

        virtual void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, I32 x, I32 y);
        virtual void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, REAL x, REAL y) { draw(drawMode, flags, (I32)x, (I32) }
        virtual void draw(IMAGE_DRAW_MODE drawMode, UI32 flags, REAL x, REAL y, const Color & /*color*/)
        {
            // TODO: color support
            draw(drawMode, flags, (I32)x, (I32)y);
        }

        // ImageSDL additions:

        inline SDL_Surface * getSDLSurface() { return mSurface; }

    protected:

        SDL_Surface *mSurface;
        UI32 mTargetWidth;
        UI32 mTargetHeight;
    }; // class ImageSDL
} // namespace turska


#endif // !T2IMAGESDL_H_INCLUDED
