/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2Types.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2TYPES_H_INCLUDED
#define T2TYPES_H_INCLUDED


namespace turska
{
    
    typedef float F32;
    typedef double F64;
    typedef signed char I8;
    typedef unsigned char UI8;
    typedef char C8;
    typedef signed short I16;
    typedef unsigned short UI16;
    typedef int I32;
    typedef unsigned int UI32;

    // Replace REAL typedef to customize for fixed-point, for example.
    typedef F32 REAL;
    
    enum GRAPHICS_ENUM
    {
        //! Use current default flags when loading images. \sa Graphics
        G_USE_DEFAULT_FLAGS = 0xffffffff
    };

    //! Defines a rectangle inside pixel image data.
    struct PixelRectangle
    {
        I16 x, y;
        I16 width, height;
    };

    //! Specifies a color as unsigned 8-bit integer values.
    struct Color
    {
        union {
            UI8 c[4];
            struct {
                UI8 r;
                UI8 g;
                UI8 b;
                UI8 a;
            };
        };

        Color() { set(0xffffffff); }
        Color(UI8 red, UI8 green, UI8 blue, UI8 alpha = 255) { set(red, green, blue, alpha); }
        Color(REAL red, REAL green, REAL blue, REAL alpha = (REAL)1.0f) { set(red, green, blue, alpha); }
        Color(I32 color) { set(color); }
        Color(UI32 color) { set(color); }
        
        inline void set(UI8 red, UI8 green, UI8 blue, UI8 alpha = 255)
        {
            r = red;
            g = green;
            b = blue;
            a = alpha;
        }
        
        inline void set(REAL red, REAL green, REAL blue, REAL alpha = (REAL)1.0f)
        {
            r = (UI8)(red * 255);
            g = (UI8)(green * 255);
            b = (UI8)(blue * 255);
            a = (UI8)(alpha * 255);
        }
        
        inline void set(UI32 color)
        {
            a = (color >> 24) & 0xff;
            r = (color >> 16) & 0xff;
            g = (color >> 8) & 0xff;
            b = color & 0xff;
        }

        inline void set(I32 color) { set((UI32)color); }
    };

    enum KEY_ACTION
    {
        KA_UNKNOWN = 0,
        KA_NEXT,     // typically tab
        KA_PREVIOUS, // typically shift-tab
        KA_LEFT,
        KA_RIGHT,
        KA_UP,
        KA_DOWN,
        KA_SELECT    // enter, kp-enter
    };
    
    struct KeyEvent
    {
        I32 character; //!< Pressed key as unicode character.
        KEY_ACTION action;
        bool shift;
        bool control;
        bool alt;
        //! Contains current platform's event structure for more details.
        /*! For SDL this is a pointer to a SDL_keysym struct.
         * For Irrlicht this is a pointer to a SKeyInput struct.
         */
        const void *platformEvent;
    };
    
} // namespace turska


#endif // !T2TYPES_H_INCLUDED
