/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2VerticalStripBitmapFont.h,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef T2VERTICALSTRIPBITMAPFONT_H_INCLUDED
#define T2VERTICALSTRIPBITMAPFONT_H_INCLUDED


namespace turska
{

    //! Defines vertical strip bitmap font related constants.
    enum VERTICALSTRIPBITMAPFONT_CONSTANTS
    {
        //! Maximum amount of glyphs.
        /*! \note Current implementations assume 8 bits per character.
         */
        VSBF_MAX_GLYPHS = 256
    };

    //! Simple bitmap font class which loads glyphs from a vertical strip.
    class VerticalStripBitmapFont : public Font
    {
    public:

        //! Constructor caches the file name and builds glyph map using font string.
        /*! \note Default font string is
         *        " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~".
         * \param fileName image file to load the font from.
         * \param fontString ordered string of characters appearing in the font image.
         */
        VerticalStripBitmapFont(const C8 *fileName, const C8 *fontString = 0);
        virtual ~VerticalStripBitmapFont();

        virtual bool init() = 0;

        virtual void setCharacterSpacing(I32 spacing) { mCharacterSpacing = spacing; }
        virtual I32 getCharacterSpacing() { return mCharacterSpacing; }
        virtual UI32 getFontHeight() { return mFontHeight; }
        
        virtual bool reload() = 0;

        virtual I32 getStringWidth(const C8 *string) = 0;

        virtual REAL getStringWidth(const C8 *string, REAL scale) = 0;

        virtual void drawString(I32 x, I32 y, const C8 *string,
                                UI32 flags = FA_DEFAULT,
                                const Color &color = Color(0xffffffff)) = 0;

        virtual void drawString(REAL x, REAL y, const C8 *string,
                                UI32 flags = FA_DEFAULT, REAL scale = 1,
                                const Color &color = Color(0xffffffff)) = 0;

    protected:

        enum VERTICALSTRIPBITMAPFONT_PRIVATE_ENUM
        {
            VSBFP_MAX_FILENAME_LENGTH = 256
        };

        void buildCharMap(const C8 *fontString);

        C8 mFileName[VSBFP_MAX_FILENAME_LENGTH];
        UI8 mGlyphMap[VSBF_MAX_GLYPHS];
        Image **mGlyphs;
        UI32 mGlyphCount;
        I32 mCharacterSpacing;
        UI32 mFontHeight;

    }; // class Font

} // namespace turska


#endif // !T2VERTICALSTRIPBITMAPFONT_H_INCLUDED
