/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUIImageButtonStyle.cpp,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#include <assert.h>

#include "Turs2.h"


namespace turska
{


IMGUIImageButtonStyle::IMGUIImageButtonStyle(Font *font, Image *imageUp, Image *imageHot,
                                             Image *imageActive, Image *imageDown) :
    mScale(1),
    mImageWidth(0), mImageHeight(0),
    mShadowEnabled(false),
    mShadowXOffset(0), mShadowYOffset(2.5f),
    mButtonDownItemXOffset(0), mButtonDownItemYOffset(2.0f)
{
    mButtonColor.set(0xffffffff);
    mItemColor.set(0xffffffff);
    mShadowMultiplierColor.set(0x80000000);
    mFont = font;
    mImages[IMGUI_WS_DEFAULT] = imageUp;
    mImages[IMGUI_WS_HOT] = imageHot;
    mImages[IMGUI_WS_ACTIVE] = imageActive;
    mImages[IMGUI_WS_HOT_ACTIVE] = imageDown;
    Image *img = mImages[IMGUI_WS_DEFAULT];
    if (img == NULL)
        img = mImages[IMGUI_WS_HOT];
    if (img == NULL)
        img = mImages[IMGUI_WS_ACTIVE];
    if (img == NULL)
        img = mImages[IMGUI_WS_HOT_ACTIVE];
    if (img != NULL)
    {
        mImageWidth = img->getWidth();
        mImageHeight = img->getHeight();
    }
}


bool IMGUIImageButtonStyle::isInside(REAL x, REAL y, REAL width, REAL height,
                                     const C8 *text, Image *image)
{
    if (mImageWidth > 0 && mImageHeight > 0)
        return x >= 0 && y >= 0 &&
               x < mImageWidth * mScale && y < mImageHeight * mScale;
    else if (image != NULL)
        return x >= 0 && y >= 0 &&
               x < image->getWidth() * mScale && y < image->getHeight() * mScale;
    else if (text != NULL)
        return x >= 0 && y >= 0 &&
               x < mFont->getStringWidth(text, mScale) &&
               y < mFont->getFontHeight() * mScale;
    else
        return x >= 0 && y >= 0 && x < width && y < height;
}


void IMGUIImageButtonStyle::render(I32 widgetState, REAL x, REAL y,
                                   REAL width, REAL height,
                                   const C8 *text, Image *image)
{
    assert(widgetState >= 0 && widgetState < IMGUI_WS_COUNT);
    Image *buttonImage = mImages[widgetState];
    UI32 defaultFlags = IF_TRANSFORM | IF_SCALE | IF_COLOR;
    if (buttonImage != NULL)
    {
        buttonImage->setScale(mScale);
        buttonImage->draw(IDM_NORMAL, defaultFlags, x, y, mButtonColor);
        width = mScale * mImageWidth;
        height = mScale * mImageHeight;
    }
    else if (image != NULL)
    {
        width = mScale * image->getWidth();
        height = mScale * image->getHeight();
    }
    else if (text != NULL && mFont != NULL)
    {
        width = mFont->getStringWidth(text, mScale);
        height = mFont->getFontHeight() * mScale;
    }

    REAL itemBaseY = y + height / 2;

    //if (buttonState == IMGUI_WS_HOT_ACTIVE)
    //    itemBaseY += mScale * 2;

    REAL bx = x + width / 2;
    REAL by = itemBaseY;

    if (mShadowEnabled)
    {
        REAL shadowX = bx + mScale * mShadowXOffset;
        REAL shadowY = by + mScale * mShadowYOffset;

        // image/text shadow
        if (image != NULL)
        {
            image->setScale(mScale);
            image->draw(IDM_NORMAL, defaultFlags | IF_CENTER,
                        shadowX, shadowY, mShadowMultiplierColor);
        }
        if (mFont != NULL && text != NULL)
        {
            mFont->drawString(shadowX, shadowY, text,
                              FA_HORIZ_CENTER | FA_VERT_CENTER,
                              mScale, mShadowMultiplierColor);
        }
    }

    // image/text itself
    if (widgetState == IMGUI_WS_HOT_ACTIVE)
    {
        bx += mScale * mButtonDownItemXOffset;
        by += mScale * mButtonDownItemYOffset;
    }
    if (image != NULL)
    {
        image->setScale(mScale);
        image->draw(IDM_NORMAL, defaultFlags | IF_CENTER,
                    bx, by, mItemColor);
    }
    if (mFont != NULL && text != NULL)
    {
        mFont->drawString(bx, by, text, FA_HORIZ_CENTER | FA_VERT_CENTER,
                          mScale, mItemColor);
    }

    // TODO: highlight text/image when hot
    if (widgetState & IMGUI_WS_MASK_HOT)
    {
    }
}


} // namespace turska
