/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2IMGUIImageSliderStyle.cpp,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#include <assert.h>

#include "Turs2.h"


namespace turska
{


IMGUIImageSliderStyle::IMGUIImageSliderStyle(Image *imageDefault, Image *imageHot,
                                             Image *imageActive, Image *imageHotActive,
                                             Image *thumbImageDefault, Image *thumbImageHot,
                                             Image *thumbImageActive, Image *thumbImageHotActive) :
    mScale(1),
    mImageWidth(30), mImageHeight(8),
    mThumbImageWidth(3), mThumbImageHeight(4)
{
    mSliderColor.set(0xffffffff);
    mThumbColor.set(0xffffffff);
    mSliderImages[IMGUI_WS_DEFAULT] = imageDefault;
    mSliderImages[IMGUI_WS_HOT] = imageHot;
    mSliderImages[IMGUI_WS_ACTIVE] = imageActive;
    mSliderImages[IMGUI_WS_HOT_ACTIVE] = imageHotActive;
    mThumbImages[IMGUI_WS_DEFAULT] = thumbImageDefault;
    mThumbImages[IMGUI_WS_HOT] = thumbImageHot;
    mThumbImages[IMGUI_WS_ACTIVE] = thumbImageActive;
    mThumbImages[IMGUI_WS_HOT_ACTIVE] = thumbImageHotActive;

    Image *img = mSliderImages[IMGUI_WS_DEFAULT];
    if (img == NULL)
        img = mSliderImages[IMGUI_WS_HOT];
    if (img == NULL)
        img = mSliderImages[IMGUI_WS_ACTIVE];
    if (img == NULL)
        img = mSliderImages[IMGUI_WS_HOT_ACTIVE];
    if (img != NULL)
    {
        mImageWidth = img->getWidth();
        mImageHeight = img->getHeight();
    }

    img = mThumbImages[IMGUI_WS_DEFAULT];
    if (img == NULL)
        img = mThumbImages[IMGUI_WS_HOT];
    if (img == NULL)
        img = mThumbImages[IMGUI_WS_ACTIVE];
    if (img == NULL)
        img = mThumbImages[IMGUI_WS_HOT_ACTIVE];
    if (img != NULL)
    {
        mThumbImageWidth = img->getWidth();
        mThumbImageHeight = img->getHeight();
    }
}


bool IMGUIImageSliderStyle::isInside(REAL x, REAL y, REAL width, REAL height)
{
    if (mSliderImages[IMGUI_WS_DEFAULT] != NULL)
        return x >= 0 && y >= 0 && x < mImageWidth * mScale && y < mImageHeight * mScale;
    else
        return x >= 0 && y >= 0 && x < width && y < height;
}


I32 IMGUIImageSliderStyle::getIntValue(IMGUI_WIDGET_ORIENTATION axis,
                                       I32 minValue, I32 maxValue,
                                       REAL x, REAL y, REAL width, REAL height)
{
    assert(axis == IMGUI_WO_HORIZONTAL || axis == IMGUI_WO_VERTICAL);
    I32 pos, length;
    REAL thumbWidth = (REAL)mThumbImageWidth, thumbHeight = (REAL)mThumbImageHeight;
    if (mSliderImages[IMGUI_WS_DEFAULT] != NULL)
    {
        width = mImageWidth * mScale;
        height = mImageHeight * mScale;
        thumbWidth = mThumbImageWidth * mScale;
        thumbHeight = mThumbImageHeight * mScale;
    }

    // TODO/FIXME - there's some small logic mistake..
    if (axis == IMGUI_WO_VERTICAL)
    {
        pos = (I32)(y - thumbHeight / 2);
        length = (I32)(height - thumbHeight);
    }
    else
    {
        // default to horizontal
        pos = (I32)(x - thumbWidth / 2);
        length = (I32)(width - thumbWidth);
    }
    if (length == 0)
        length = 1;

    I32 fromRangeRound = length / 2;
    I32 toValueRange = maxValue - minValue + 1;
    I32 result = (pos * toValueRange + fromRangeRound) / length;
    result += minValue;

    if (result < minValue)
        result = minValue;
    if (result > maxValue)
        result = maxValue;

    return result;
}


REAL IMGUIImageSliderStyle::getRealValue(IMGUI_WIDGET_ORIENTATION axis,
                                         REAL minValue, REAL maxValue,
                                         REAL x, REAL y, REAL width, REAL height)
{
    assert(axis == IMGUI_WO_HORIZONTAL || axis == IMGUI_WO_VERTICAL);
    REAL pos, length;
    REAL thumbWidth = (REAL)mThumbImageWidth, thumbHeight = (REAL)mThumbImageHeight;
    if (mSliderImages[IMGUI_WS_DEFAULT] != NULL)
    {
        width = mImageWidth * mScale;
        height = mImageHeight * mScale;
        thumbWidth = mThumbImageWidth * mScale;
        thumbHeight = mThumbImageHeight * mScale;
    }

    // TODO/FIXME - there's some small logic mistake..
    if (axis == IMGUI_WO_VERTICAL)
    {
        pos = y - thumbHeight / 2;
        length = height - thumbHeight;
    }
    else
    {
        // default to horizontal
        pos = x - thumbWidth / 2;
        length = width - thumbWidth;
    }
    if (length == 0)
        length = 1;

    REAL toValueRange = maxValue - minValue;
    REAL result = (pos * toValueRange) / length;
    result += minValue;

    if (result < minValue)
        result = minValue;
    if (result > maxValue)
        result = maxValue;

    return result;
}


void IMGUIImageSliderStyle::render(I32 widgetState, IMGUI_WIDGET_ORIENTATION axis,
                                   I32 value, I32 minValue, I32 maxValue,
                                   REAL x, REAL y, REAL width, REAL height)
{
    REAL normalizedValue;

    const REAL fromRange = (REAL)(maxValue - minValue);
    normalizedValue = (value - minValue) / fromRange;

    render(widgetState, axis, normalizedValue, x, y, width, height);
}


void IMGUIImageSliderStyle::render(I32 widgetState, IMGUI_WIDGET_ORIENTATION axis,
                                   REAL value, REAL minValue, REAL maxValue,
                                   REAL x, REAL y, REAL width, REAL height)
{
    REAL normalizedValue;

    const REAL fromRange = maxValue - minValue;
    normalizedValue = (value - minValue) / fromRange;

    render(widgetState, axis, normalizedValue, x, y, width, height);
}


void IMGUIImageSliderStyle::render(I32 widgetState, IMGUI_WIDGET_ORIENTATION axis,
                                   REAL normalizedValue,
                                   REAL x, REAL y, REAL width, REAL height)
{
    assert(widgetState >= 0 && widgetState < IMGUI_WS_COUNT);
    Image *sliderImage = mSliderImages[widgetState];
    UI32 defaultFlags = IF_TRANSFORM | IF_SCALE | IF_COLOR;
    REAL thumbWidth = (REAL)mThumbImageWidth, thumbHeight = (REAL)mThumbImageHeight;
    if (sliderImage != NULL)
    {
        sliderImage->setScale(mScale);
        sliderImage->draw(IDM_NORMAL, defaultFlags, x, y, mSliderColor);
        width = mImageWidth * mScale;
        height = mImageHeight * mScale;
        thumbWidth = mThumbImageWidth * mScale;
        thumbHeight = mThumbImageHeight * mScale;
    }

    REAL thumbX = x, thumbY = y;
    if (axis == IMGUI_WO_VERTICAL)
    {
        REAL toRange = height - thumbHeight;
        thumbX += width / 2;
        thumbY += normalizedValue * toRange + thumbHeight / 2;
    }
    else
    {
        // default to horizontal
        REAL toRange = width - thumbWidth;
        thumbX += normalizedValue * toRange + thumbWidth / 2;
        thumbY += height / 2;
    }

    // thumb
    Image *thumbImage = mThumbImages[widgetState];
    if (thumbImage != NULL)
    {
        thumbImage->setScale(mScale);
        thumbImage->draw(IDM_NORMAL, defaultFlags | IF_CENTER,
                         thumbX, thumbY, mThumbColor);
    }
}


} // namespace turska
