/* Turska Framework/UI library (Turs2)
 * Copyright 2003-2008 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/turska/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: T2VerticalStripBitmapFont.cpp,v 1.1 2008-04-19 11:46:25 tonic Exp $
 * $Revision: 1.1 $
 */

#include <assert.h>

#include "Turs2.h"


namespace turska
{


const static C8 sDefaultFontStr[] =
    " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";


VerticalStripBitmapFont::VerticalStripBitmapFont(const C8 *fileName,
                                                 const C8 *fontString) :
    mGlyphs(0), mGlyphCount(0),
    mCharacterSpacing(1), mFontHeight(0)
{
    memset(mFileName, 0, VSBFP_MAX_FILENAME_LENGTH);
    memset(mGlyphMap, 0, VSBF_MAX_GLYPHS);

    strncpy(mFileName, fileName, VSBFP_MAX_FILENAME_LENGTH);

    if (fontString == NULL)
        fontString = sDefaultFontStr;
    buildCharMap(fontString);
}


VerticalStripBitmapFont::~VerticalStripBitmapFont()
{
    UI32 a;
    for (a = 0; a < mGlyphCount; ++a)
        delete mGlyphs[a];
    delete[] mGlyphs;
    mGlyphs = NULL;
    mGlyphCount = 0;
    mLoaded = false;
}


void VerticalStripBitmapFont::buildCharMap(const C8 *fontString)
{
    UI32 a, ch, mapped;

    mGlyphCount = (UI32)strlen(fontString);

    for (ch = 0; ch < VSBF_MAX_GLYPHS; ++ch)
    {
        /* All chars map to first glyph by default
         * if different glyph isn't specified.
         */
        mapped = 0;

        for (a = 0; a < mGlyphCount; ++a)
        {
            if (fontString[a] == (C8)ch)
            {
                mapped = a;
                break;
            }
        }

        mGlyphMap[ch] = (UI8)mapped;
    }
}


} // namespace turska
